/**
 * This file is part of GenomeView, a genome browser and annotation curator
 * 
 * Copyright (C) 2007-2013 Thomas Abeel and contributors
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 * Project: http://genomeview.org/
 */
package net.sf.genomeview.data.provider;

import net.sf.genomeview.data.GenomeViewScheduler;


/**
 * Data provider for visualization tracks. 
 * 
 * Methods should return immediately,
 * data fetching should be done in background threads.
 * 
 * @see GenomeViewScheduler
 * 
 * @author Thomas Abeel
 * 
 * @param <T>
 */
public interface DataProvider<T> {

	/**
	 * Get data. The selected data should cover [start,end[. The coordinates
	 * are one based.
	 * 
	 * @param start
	 *            the start coordinate, this one will be included. This is a
	 *            one-based coordinate.
	 * @param end
	 *            the end coordinate, this one will not be included. This is a
	 *            one-based coordinate.
	 * @return the selected data.
	 */
	public void get(int start, int end,DataCallback<T> cb);

//	public Iterable<Status> getStatus(int start, int end);


}
