/**
 * This file is part of GenomeView, a genome browser and annotation curator
 * 
 * Copyright (C) 2007-2013 Thomas Abeel and contributors
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 * Project: http://genomeview.org/
 */
package net.sf.genomeview.gui;

import net.sf.jannot.Location;

public class Convert {
	/**
	 * Translates a coordinate on the genome to a screen coordinate. This is
	 * only needed for the X coordinate
	 * 
	 * @param g
	 *            the graphics context for the visible Location
	 * @param pos
	 *            the position to translate, this may be outside the visible
	 *            Location
	 * @return
	 */
	public static int translateGenomeToScreen(int pos, Location r,double screenWidth) {
		int genomeWidth = r.length();
		int relativePos = pos - r.start;
		return (int) (screenWidth / genomeWidth * relativePos);

	}

	/**
	 * Translates a coordinate on the screen to a genome coordinate. This is
	 * only needed for the X coordinate
	 * 
	 * @param g
	 *            the graphics context for the visible Location
	 * @param pos
	 *            the position to translate,
	 * 
	 * @return
	 */
	public static int translateScreenToGenome(int pos, Location r,double screenWidth) {
		// Location r = model.getAnnotationLocationVisible();
		int genomeWidth = r.length();
		return (int) (pos / screenWidth * genomeWidth) + r.start;

	}
}
