/**
 * This file is part of GenomeView, a genome browser and annotation curator
 * 
 * Copyright (C) 2007-2013 Thomas Abeel and contributors
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 * Project: http://genomeview.org/
 */
package net.sf.genomeview.gui.config;

import java.awt.Color;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.JColorChooser;
import javax.swing.JLabel;

import net.sf.genomeview.core.ColorIcon;
import net.sf.genomeview.core.Configuration;
import net.sf.genomeview.data.Model;
import be.abeel.gui.GridBagPanel;

public class ColorConfig extends GridBagPanel {

		private static final long serialVersionUID = -2242613993844951737L;

		public ColorConfig(Model model, String key, String msg) {
			gc.weightx=0;
			gc.weighty=0;
			add(new JLabel(msg), gc);
			gc.gridx++;
			add(new ColorLabel(model, key), gc);
		}

		private class ColorLabel extends JLabel {

			private static final long serialVersionUID = -290128964356729238L;

			private ColorLabel(final Model model, final String configKey) {
				super(new ColorIcon(Configuration.getColor(configKey), 16));
				this.addMouseListener(new MouseAdapter() {
					public void mouseClicked(MouseEvent e) {
						Color newColor = JColorChooser.showDialog(model.getGUIManager().getParent(), "Choose color", Configuration.getColor(configKey));

						if (newColor != null) {
							Configuration.setColor(configKey, newColor);
							setIcon(new ColorIcon(Configuration.getColor(configKey), 16));
						}
					}
				});

			}
		}

	}
