/**
 * This file is part of GenomeView, a genome browser and annotation curator
 * 
 * Copyright (C) 2007-2013 Thomas Abeel and contributors
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 * Project: http://genomeview.org/
 */
package net.sf.genomeview.gui.dialog;

import java.awt.Dimension;
import java.awt.EventQueue;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Window;

import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JProgressBar;

import net.sf.genomeview.gui.StaticUtils;

/**
 * A popup dialog containing a progress bar and progress messages. It is
 * designed to be passed as a PropertyChangeListener to SwingWorkers.
 * 
 * @author thpar
 * @author Thomas Abeel
 */
public class GVProgressBar extends JDialog {

	private static final long serialVersionUID = -84766714924209282L;

	private JLabel commentLabel = new JLabel();

	private JProgressBar pb = new JProgressBar();

	public GVProgressBar(String title, String initText, Frame parent) {
		super(parent, title,ModalityType.MODELESS);
		requestFocus();
		setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
		setLayout(new GridBagLayout());
		setPreferredSize(new Dimension(280, 80));
		pb.setIndeterminate(true);
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.fill = GridBagConstraints.BOTH;
		gbc.anchor = GridBagConstraints.CENTER;
		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.gridy = GridBagConstraints.RELATIVE;
		commentLabel.setText(initText);
		add(commentLabel, gbc);
		add(pb, gbc);
		pack();
		StaticUtils.upperRight(this);

		setVisible(true);
	}

	public void done() {
		final Window jf = this;
		EventQueue.invokeLater(new Runnable() {

			@Override
			public void run() {
				jf.setVisible(false);
				jf.dispose();
			}

		});
		
	}

	public void setComment(String comment) {
		commentLabel.setText(comment);

	}

	public void setMax(int max) {
		pb.setIndeterminate(false);
		pb.setMaximum(max);
	}

	public void setProgress(int progress) {
		pb.setValue(progress);

	}

	public void inc() {
		pb.setValue(pb.getValue() + 1);

	}

}
